c *********************************************************************
c   version 8.0:  with second order degree four terms
c   with nonlinear forced terms for eccentricity
c    s7 terms in degree four long periodic terms 
c *********************************************************************
      program secre8
      implicit double precision (a-h,o-z)
      include 'control.h'
c  accuracy of the computation of Laplace coefficients
      parameter (excrit=1.d-6)
c  forced terms for the asteroid xi_j, eta_j; j frequency index
      dimension xi(8),et(8),xip(6),etp(6)
c  corrections to the frequencies (for each planet)
      dimension cg(8),cs(8)
c  control for perturbing planets included and order of theory
c      dimension ipla(8)
c  secular theory constants for the planets
c  one-dim arrays; index is frequency index lambda_j, delta_j
c  nu_j, mu_j; j is frequency index for the current planet
      dimension pl(8),pd(8),ppni(8),ppmi(8)
c  secular theory frequencies for the planets g_j, s_j
      dimension pg(8),ps(8)
c  two-dim arrays; first index is planet index, second is freq. index
c nu_{j,k}, mu_{j,k} with j planet ind, k freq. index
      dimension pmi(8,8),pni(8,8)
c  masses, mean motions and semimajor axes of the planets
      dimension pmv(8),pn(8),pa(8)
c  yuasa coefficients  a and b (first order effects)
c  warning: here planet index is the second one, first is Yuasa subscript
      dimension yya(4,8),yyb(27,8)
c  only for lonopt
      parameter (nnlx=20)
c  nonlinear forced terms: frequencies, phases, amplitudes, forced ecc.
      dimension fnl(nnlx),phanl(nnlx),pninl(2,nnlx),xinl(nnlx)
c      dimension dd(59)
c  header skip device, end of file, version  and pi greek
c      character*60 vers
      common/trig/dpig
c  mean elements to be used for discarded cases
      common/meanels/ae,ai0,aw0,ao0
c  constants
      pik=4.d0*datan(1.d0)
      dpig=2.d0*pik
c
c *********************************************************************
c
c  read controls from file secre8.opt
c
c *********************************************************************
      open(1,file='secre8.opt')
      call gridsel(ngrid,ita,a0,delta,ite,e0,delte,
     $      iti,ai0,delti,aw,ao)
      deltir=delti*pik/1.8d2
      ai0r= ai0*pik/1.8d2
      open(3,file='datam.m')
      write(3,555)ngrid,a0,delta,ai0,delti,e0,delte,aw,ao
555   format(' data=[',i5,1x,2(f10.7,1x),4(f10.7,1x),2(f10.4,1x),']'/)
c  version-dependent options, defining the theory used (see optxx)
c  ive < 0 Euler; ive=3 RK2 explicit (see inte8)
      inp=1
      ive=-1
      call lonopt(inp,ive)
c      call lonopt(inp,fconte,fconti,nitmax,divrat,ipla,isec,
c     +      nplin,nplou,nplin0,nplou0,npli4,nplo4,dd,divm,iqcr,vers)
c
c *********************************************************************
c
c  initialisations, file opening, etc.
c
c **********************************************************************
      open(2,file='gs.fla',status='unknown')
      open(4,file='forced.fla',status='unknown')
c
c ********************************************************************
c  the secular perturbation theory has to be consulted to find the
c  appropriate semimajor axis of the planets, and later the secular
c  eccentricities and inclinations will be used as well as the
c  fundamental secular angles (see secthe67)
c  the fixed time is defined by t0 (in JD), with 0 shift
      t=0.d0
c      isec=1
      call secthe(nplin,nplou,t,t0,pmv,pa,pni,pg,pl,pmi,ps,pd
     +      ,nnl,fnl,pninl,phanl,isec)
c  data on the inner planets for computation of g0, s0
      call inplda(pa,pmv)
c *******************************************************************
c change of scale, units, fundamental constants
      call convun(pmv,pa,pg,ps,pn,aunit,tunit,pmunit,nplin0,nplou
     +      ,fnl,nnl)
c square root of solar mass
      sm=dsqrt(1.d0/pmunit)
c  conversion factor to get frequencies in arcsec/year
      conv=1.d0/tunit*180.d0/pik*3600.d0
c *********************************************************************
c
c  computations depending on semimajor axis only
c
c **********************************************************************
      do 300 it3=1,ita
      aa=a0+delta*(it3-1)
      a=aa/aunit
      al=sm*dsqrt(a)
      sl=dsqrt(al)
c  asteroid mean motion
      an=sm/al**3
c *********************************************************************
c   laplace's, leverrier's and yuasa's coefficients.
c *********************************************************************
      do 27 j=nplin0,nplou
 27     call llyco(a,pa(j),an,pn(j),sm,yya(1,j),yyb(1,j),pmv(j)
     +     ,ipla(j),excrit)
c *********************************************************************
c   second order/degree frequencies and forced oscillation amplitudes.
c *********************************************************************
      call g0s0(al,yya,pmv,g0,s0,nplin0,nplou0)
c   forced oscillation amplitudes; from outer planets only
      call forced(g0,s0,pg,ps,pmv,pni,pmi,yya,al,xi,et,
     +    nplin,nplou)
c *********************************************************************
c   computations depending on eccentricity and inclinations
c *********************************************************************
      do 400 it2=1,iti
      ai=ai0r+(it2-1)*deltir
      do 500 it1=1,ite
      ae=e0+(it1-1)*delte
c  counter of grid points
      iter=iter+1
c
c  delaunay's and poincare's variables
c
      ag=al*dsqrt(1.d0-ae*ae)
      ah=ag*dcos(ai)
      amp1=max(2.d0*(al-ag),0.d0)
      ap=dsqrt(amp1)*dcos(aw+ao)
      aq=-dsqrt(amp1)*dsin(aw+ao)
      amp2=max(2.d0*(ag-ah),0.d0)
      ar=dsqrt(amp2)*dcos(ao)
      as=-dsqrt(amp2)*dsin(ao)
c
c  proper modes.
c
      ro1=ap/sl
      ro2=-aq/sl
      sig1=ar/sl
      sig2=-as/sl
c
c proper actions (doubled)
c
      ani=ro1**2+ro2**2
      ami=sig1**2+sig2**2

c *********************************************************************
c
c  Degree 4 corrections to the frequencies, a function of the proper
c  amplitudes ami, ani and of the forced terms xi, eta.
c  In this version the perturbing function for planets from npli4 to nplo4
c  is taken into account to degree 4; however both for the planets and for
c  the asteroid the forced terms with all the frequencies
c  (from nplin to nplou) are accounted for.
c
c *********************************************************************
      g=g0
      s=s0
c  loop on planets
      do 51 jp=npli4,nplo4
c  proper modes of current perturbing planet
        do 50 nfr=nplin,nplou
          ppni(nfr)=pni(jp,nfr)
          ppmi(nfr)=pmi(jp,nfr)
 50     continue
        call gsfour(pmv(jp),yya(1,jp),yyb(1,jp),xi,et,ppni,ppmi,
     *         ani,ami,cg(jp),cs(jp),nplin,nplou)
        g=g+2.d0/al*cg(jp)
        s=s+2.d0/al*cs(jp)
 51   continue
c   forced oscillation amplitudes; now with new frequencies g,s
      call forced(g,s,pg,ps,pmv,pni,pmi,yya,al,xip,etp,
     +    nplin,nplou)
c *********************************************************************
c
c  conversion and output
c
      gp=g*conv
      sp=s*conv
      if(gp.ge.1.d3)gp=999.999d0
      if(gp.le.-1.d3)gp=-999.999d0
      if(sp.ge.1.d3)sp=999.999d0
      if(sp.le.-1.d3)sp=-999.999d0
      write(2,601)gp,sp
 601  format(2f12.4)
c
      if(xip(6).gt.1.d0)then
            xip(6)=1.d0
      elseif(xip(6).lt.-1.d0)then
            xip(6)=-1.d0
      endif
      if(etp(6).gt.1.d0)then
            etp(6)=1.d0
      elseif(etp(6).lt.-1.d0)then
            etp(6)=-1.d0
      endif
      write(4,602)xip(6),etp(6)
 602  format(2f10.6)
c *********************************************************************
c
c   end of grid loops
c
 500  continue
 400  continue
c     write(*,*)it3
 300  continue
      n=iter
      close(2)
      close(4)
      stop
      end




